//
//  ECellBLEDataParse.m
//  Bluetooth_Example
//
//  Created by lemo. on 2019/5/25.
//  Copyright © 2019 chentingfeng. All rights reserved.
//

#import "ECellBLEDataParse.h"

// 运动数据新格式
#define KEY_SPORTMODEL_SPORT 0xAA
#define KEY_SPORTMODEL_GPS 0xB0
#define KEY_SPORTMODEL_KM 0xB1
#define KEY_SPORTMODEL_STEP 0xB2
#define KEY_SPORTMODEL_HRS 0xB3
#define KEY_SPORTMODEL_SWIM 0xB4

@implementation ECellBLEDataParse


+ (NSDictionary *)parse:(NSMutableData *)data {
    ECell_BLECMD_KEY cmdKey = [[NSString stringWithFormat:@"%02d", ((Byte*)([data bytes]))[10]] integerValue];
    return [ECellBLEDataParse parse:data cmd:cmdKey];
}

/// 数据解析
/// @param data 完整数据
/// @param cmd 指令码KEY
+ (NSDictionary *)parse:(NSMutableData *)data cmd:(ECell_BLECMD_KEY)cmd{
    
    NSMutableDictionary *mutableDict = [[NSMutableDictionary alloc]init];
    //  是否存在内容
    BOOL isContent = data.length > 13 ? YES:NO;
    if (!isContent) {
        return mutableDict;
    }
    
    NSLog(@"cmd=====%@===%ld", [self ToHex:cmd],cmd);
    NSLog(@"data=====%@", [self convertDataToHexStr:data]);
    
   
    
    switch (cmd) {
        case ECell_BLECMD_KEY_syncHealth:
        case ECell_BLECMD_KEY_step_return:
        case ECell_BLECMD_KEY_sleep_return:
        case ECell_BLECMD_KEY_heartRate_return:
        case ECell_BLECMD_KEY_sport_return:
        {
            mutableDict = [self p_parseBraceletData_syncHealthWith:data];
        }
            break;
            // 固件信息
        case ECell_BLECMD_KEY_firmwareInfo:
        case ECell_BLECMD_KEY_firmwareInfo_return:
        {
            Byte *buffer = (Byte *)[data bytes];
            NSString *deviceName = [[NSString alloc] initWithData:[data subdataWithRange:NSMakeRange(13, 10)] encoding:NSASCIIStringEncoding];
            NSInteger platform = [[[NSString alloc] initWithFormat:@"%2d",buffer[23]] integerValue];
            NSString *macAddress = [self getPositiveSequenceMacWithData:[data subdataWithRange:NSMakeRange(24, 6)]];
            NSInteger adapterNumber = [[[self class] HexToUnsignedInt:data dataRange:NSMakeRange(30, 3)] integerValue];
            NSString *protocolVersion = [[NSString alloc] initWithFormat:@"%d.%d",buffer[33],buffer[34]];
            NSString *firmwareVersion = [[NSString alloc] initWithFormat:@"%d.%d.%d",buffer[35],buffer[36],buffer[37]];
            [mutableDict setObject:deviceName forKey:@"deviceName"];
            [mutableDict setObject:@(platform) forKey:@"platform"];
            [mutableDict setObject:macAddress forKey:@"macAddress"];
            [mutableDict setObject:@(adapterNumber) forKey:@"adapterNumber"];
            [mutableDict setObject:protocolVersion forKey:@"protocolVersion"];
            [mutableDict setObject:firmwareVersion forKey:@"firmwareVersion"];
            [mutableDict setObject:@(0x13) forKey:@"cmd"];
            NSLog(@"获取固件信息-->%ld",cmd);
            
        }
            break;
            // 电量返回
        case ECell_BLECMD_KEY_battery:
        case ECell_BLECMD_KEY_battery_return:
        {
            Byte *buffer = (Byte *)[data bytes];
            NSInteger power = [[[NSString alloc] initWithFormat:@"%02d",buffer[13]] integerValue];
            NSInteger status = [[[NSString alloc] initWithFormat:@"%02d",buffer[14]] integerValue];
            [mutableDict setObject:@(power) forKey:@"power"];
            [mutableDict setObject:@(status) forKey:@"status"];
            [mutableDict setObject:@(cmd) forKey:@"cmd"];
        }
            break;
            // 当天数据
        case ECell_BLECMD_KEY_realTimesync:
        {
            Byte *buffer = (Byte *)[data bytes];
            Byte key = buffer[10];
            mutableDict = [self parseRealTimeData:data cmdKey:key];
        }
            break;
            //  读取手环设置请求
        case ECell_BLECMD_KEY_readBraceletSet:
        {
            if (data.length == 19) {
                NSData *MACData = [data subdataWithRange:NSMakeRange(11, 8)];
                NSString *macAddress = [self getPositiveSequenceMacWithData:MACData];
                [mutableDict setObject:macAddress forKey:@"macAddress"];
                [mutableDict setObject:@(cmd) forKey:@"cmd"];
            }
        }
            break;
            
        case ECell_BLECMD_KEY_readProfile:      //  读取手环配置请求
        case ECell_BLECMD_KEY_readProfile_return:
        {
            //  长度不够返回
            BOOL is87 = (data.length == 87);
            BOOL is88 = (data.length == 88);
            BOOL is106 = (data.length == 106);
        
            if (is87||is88||is106) {
                //  闹钟 5组25bytes
                Byte *alarmClockByte = (Byte *)[[data subdataWithRange:NSMakeRange(13, 25)] bytes];
                NSMutableArray *alarmClockArr = [[NSMutableArray alloc] init];
                for (NSInteger i = 0; i < 5; i ++) {
                    NSMutableDictionary *alarmClockDict = [[NSMutableDictionary alloc] init];
                    NSString *hour = [NSString stringWithFormat:@"%02d",alarmClockByte[0 + i * 5]];
                    NSString *minutes = [NSString stringWithFormat:@"%02d",alarmClockByte[1 + i * 5]];
//                    NSString *cycle = [[self class] cycleIntToString:alarmClockByte[2 + i * 5]];
                    NSString *cycle = [NSString stringWithFormat:@"%02d",alarmClockByte[2 + i * 5]];
                    NSString *label = [NSString stringWithFormat:@"%d",alarmClockByte[3 + i * 5]];
                    NSString *isOpen = [NSString stringWithFormat:@"%d",alarmClockByte[4 + i * 5]];
                    [alarmClockDict setObject:@(hour.integerValue) forKey:@"hour"];
                    [alarmClockDict setObject:@(minutes.integerValue) forKey:@"min"];
                    [alarmClockDict setObject:@(cycle.integerValue)  forKey:@"cycle"];
                    [alarmClockDict setObject:label forKey:@"label"];
                    [alarmClockDict setObject:@(isOpen.boolValue) forKey:@"isOpen"];
                    [alarmClockArr addObject:alarmClockDict];
                }
                [mutableDict setObject:alarmClockArr forKey:@"alarmClock"];
                //  久坐提醒 8bytes
                Byte *sedentaryByte = (Byte *)[[data subdataWithRange:NSMakeRange(38, 8)] bytes];
                NSMutableDictionary *sedentaryDict = [[NSMutableDictionary alloc] init];
                NSString *isOpen = [NSString stringWithFormat:@"%d",sedentaryByte[0]];
                NSString *startHour = [NSString stringWithFormat:@"%02d",sedentaryByte[1]];
                NSString *endHour = [NSString stringWithFormat:@"%02d",sedentaryByte[2]];
//                NSString *cycle = [[self class] cycleIntToString:sedentaryByte[3]];
                NSString *cycle = [NSString stringWithFormat:@"%02d",sedentaryByte[3]];
                NSString *sittingTime = [[self class]HexToUnsignedInt:data dataRange:NSMakeRange(42, 2)];
                NSString *sittingThreshold = [[self class]HexToUnsignedInt:data dataRange:NSMakeRange(44, 2)];
                [sedentaryDict setObject:@(isOpen.boolValue) forKey:@"isOpen"];
                [sedentaryDict setObject:@(startHour.integerValue) forKey:@"startHour"];
                [sedentaryDict setObject:@(endHour.integerValue) forKey:@"endHour"];
                [sedentaryDict setObject:@(cycle.integerValue) forKey:@"cycle"];
                [sedentaryDict setObject:@(sittingTime.integerValue) forKey:@"sittingTime"];
                [sedentaryDict setObject:@(sittingThreshold.integerValue) forKey:@"sittingThreshold"];
                [mutableDict setObject:sedentaryDict forKey:@"sedentary"];
                //  用户信息 8bytes
                Byte *userInfoByte = (Byte *)[[data subdataWithRange:NSMakeRange(46, 8)] bytes];
                NSMutableDictionary *userInfoDict = [[NSMutableDictionary alloc] init];
                NSString *gender = [NSString stringWithFormat:@"%d",userInfoByte[0]];
                NSString *age = [NSString stringWithFormat:@"%d",userInfoByte[1]];
                NSString *height = [NSString stringWithFormat:@"%d",userInfoByte[2]];
                NSString *weight = [NSString stringWithFormat:@"%d",userInfoByte[3]];
                NSString *goal = [[self class]HexToUnsignedIntLittle:data dataRange:NSMakeRange(50, 3)];
                NSString *sportMode = [NSString stringWithFormat:@"%d",userInfoByte[7]];
                [userInfoDict setObject:@(gender.integerValue) forKey:@"gender"];
                [userInfoDict setObject:@(age.integerValue) forKey:@"age"];
                [userInfoDict setObject:@(height.integerValue) forKey:@"height"];
                [userInfoDict setObject:@(weight.integerValue) forKey:@"weight"];
                [userInfoDict setObject:@(goal.integerValue) forKey:@"goalStep"];
                [userInfoDict setObject:@(sportMode.integerValue) forKey:@"sportMode"];
                [mutableDict setObject:userInfoDict forKey:@"userInfo"];
                //  提醒模式 1bytes
                Byte *remindModeByte = (Byte *)[[data subdataWithRange:NSMakeRange(54, 1)] bytes];
                NSMutableDictionary *remindModeDict = [[NSMutableDictionary alloc] init];
                NSString *remindType = [NSString stringWithFormat:@"%d",remindModeByte[0]];
                [remindModeDict setObject:@(remindType.intValue) forKey:@"remindType"];
                [mutableDict setObject:remindModeDict forKey:@"remindMode"];
                //  免打扰设置 5bytes
                Byte *notDisturbByte = (Byte *)[[data subdataWithRange:NSMakeRange(55, 5)] bytes];
                NSMutableDictionary *notDisturbDict = [[NSMutableDictionary alloc] init];
                NSString *isON = [NSString stringWithFormat:@"%d",notDisturbByte[0]];
                NSString *startTime = [NSString stringWithFormat:@"%@:%@",[NSString stringWithFormat:@"%02d",notDisturbByte[1]],[NSString stringWithFormat:@"%02d",notDisturbByte[2]]];
                NSString *endTime = [NSString stringWithFormat:@"%@:%@",[NSString stringWithFormat:@"%02d",notDisturbByte[3]],[NSString stringWithFormat:@"%02d",notDisturbByte[4]]];
                [notDisturbDict setObject:@(isON.boolValue) forKey:@"isOpen"];
                [notDisturbDict setObject:@(startTime.integerValue) forKey:@"startHour"];
                [notDisturbDict setObject:@(endTime.integerValue) forKey:@"endHour"];
                [mutableDict setObject:notDisturbDict forKey:@"notDisturb"];
                
                //  心率检测 6bytes
                Byte *heartRateByte = (Byte *)[[data subdataWithRange:NSMakeRange(60, 6)] bytes];
                NSMutableDictionary *heartRateDict = [[NSMutableDictionary alloc] init];
                NSString *ONOFF = [NSString stringWithFormat:@"%d",heartRateByte[0]];
                NSString *heartStartTime = [NSString stringWithFormat:@"%@:%@",[NSString stringWithFormat:@"%02d",heartRateByte[1]],[NSString stringWithFormat:@"%02d",heartRateByte[2]]];
                NSString *heartEndTime = [NSString stringWithFormat:@"%@:%@",[NSString stringWithFormat:@"%02d",heartRateByte[3]],[NSString stringWithFormat:@"%02d",heartRateByte[4]]];
                NSString *interval = [NSString stringWithFormat:@"%d",heartRateByte[5]];
                [heartRateDict setObject:@(ONOFF.boolValue) forKey:@"isOpen"];
                [heartRateDict setObject:@(heartStartTime.integerValue) forKey:@"startHour"];
                [heartRateDict setObject:@(heartEndTime.integerValue) forKey:@"endHour"];
                [heartRateDict setObject:@(interval.integerValue) forKey:@"interval"];
                [mutableDict setObject:heartRateDict forKey:@"heartRateCheck"];
                
                //  系统设置 4bytes(不处理)
                
                //  喝水提醒 8bytes
                Byte *drinkRemindByte = (Byte *)[[data subdataWithRange:NSMakeRange(70, 8)] bytes];
                NSMutableDictionary *drinkRemindDict = [[NSMutableDictionary alloc] init];
                NSString *isONOFF = [NSString stringWithFormat:@"%d",drinkRemindByte[0]];
                NSString *drinkStartTime = [NSString stringWithFormat:@"%@:%@",[NSString stringWithFormat:@"%02d",drinkRemindByte[1]],[NSString stringWithFormat:@"%02d",drinkRemindByte[2]]];
                NSString *drinkEndTime = [NSString stringWithFormat:@"%@:%@",[NSString stringWithFormat:@"%02d",drinkRemindByte[3]],[NSString stringWithFormat:@"%02d",drinkRemindByte[4]]];
//                NSString *drinkCycle = [[self class] cycleIntToString:drinkRemindByte[5]];
                NSString *drinkCycle = [NSString stringWithFormat:@"%d",drinkRemindByte[5]];
                NSString *drinkInterval = [[self class]HexToUnsignedInt:data dataRange:NSMakeRange(76, 2)];
                [drinkRemindDict setObject:@(isONOFF.boolValue) forKey:@"isOpen"];
                [drinkRemindDict setObject:@(drinkStartTime.integerValue) forKey:@"startHour"];
                [drinkRemindDict setObject:@(drinkEndTime.integerValue) forKey:@"endHour"];
                [drinkRemindDict setObject:@(drinkCycle.integerValue) forKey:@"cycle"];
                [drinkRemindDict setObject:@(drinkInterval.integerValue) forKey:@"interval"];
                [mutableDict setObject:drinkRemindDict forKey:@"drinkRemind"];
                
                //  消息推送 2bytes
                NSMutableDictionary *setPushDict = [[NSMutableDictionary alloc] init];
                NSString *pushIsON = [[self class]HexToUnsignedInt:data dataRange:NSMakeRange(78, 2)];
                [setPushDict setObject:@(pushIsON.intValue) forKey:@"isOpen"];
                [mutableDict setObject:setPushDict forKey:@"setPush"];
                
                //  运动目标 4bytes
                Byte *targetByte = (Byte *)[[data subdataWithRange:NSMakeRange(80, 4)] bytes];
                NSMutableDictionary *targetDict = [[NSMutableDictionary alloc] init];
                NSString *targetSteps = [[self class]HexToUnsignedIntLittle:data dataRange:NSMakeRange(80, 3)];
                NSString *targetSportMode = [NSString stringWithFormat:@"%d",targetByte[3]];
                [targetDict setObject:@(targetSteps.integerValue) forKey:@"targetSteps"];
                [targetDict setObject:@(targetSportMode.integerValue) forKey:@"sportMode"];
                [mutableDict setObject:targetDict forKey:@"target"];
                
                //  抬手亮屏 3bytes
                Byte *gesturesByte = (Byte *)[[data subdataWithRange:NSMakeRange(84, 3)] bytes];
                NSMutableDictionary *gesturesDict = [[NSMutableDictionary alloc] init];
                NSString *hand = [NSString stringWithFormat:@"%d",gesturesByte[0]];
                NSString *raise = [NSString stringWithFormat:@"%d",gesturesByte[1]];
                NSString *flip = [NSString stringWithFormat:@"%d",gesturesByte[2]];
                [gesturesDict setObject:@(hand.integerValue) forKey:@"hand"];
                [gesturesDict setObject:@(raise.integerValue) forKey:@"raise"];
                [gesturesDict setObject:@(flip.integerValue) forKey:@"flip"];
                [mutableDict setObject:gesturesDict forKey:@"handUp"];
                //  呼吸 1bytes
                if (is88) {//如果总字节数有88个字节，就有呼吸
                    //  呼吸 1bytes
                    Byte *breatheByte = (Byte *)[[data subdataWithRange:NSMakeRange(87, 1)] bytes];
                    NSMutableDictionary *breatheDict = [[NSMutableDictionary alloc] init];
                    NSString *breatheTime = [NSString stringWithFormat:@"%d",breatheByte[0]];
                    [breatheDict setObject:@(breatheTime.intValue) forKey:@"breatheTime"];
                    [mutableDict setObject:breatheDict forKey:@"breathe"];
                }
                
                if (is106) {
                    // 静坐调息 1bytes
                    Byte *sitByte = (Byte *)[[data subdataWithRange:NSMakeRange(88, 1)] bytes];
                    NSMutableDictionary *sitDict = [[NSMutableDictionary alloc] init];
                    NSString *sitTime = [NSString stringWithFormat:@"%d",sitByte[0]];
                    [sitDict setObject:@(sitTime.intValue) forKey:@"sitByteTime"];
                    [mutableDict setObject:sitDict forKey:@"sit"];
                    
                    //睡眠开始结束时间 6bytes
                    Byte *sleepByte = (Byte *)[[data subdataWithRange:NSMakeRange(89, 6)] bytes]; //95
                    NSMutableDictionary *sleepDict = [[NSMutableDictionary alloc] init];
                    NSString *startHour = [NSString stringWithFormat:@"%d",sleepByte[0]];
                    NSString *startMinute = [NSString stringWithFormat:@"%d",sleepByte[1]];
                    NSString *startSecond = [NSString stringWithFormat:@"%d",sleepByte[2]];
                    NSString *endHour = [NSString stringWithFormat:@"%d",sleepByte[3]];
                    NSString *endMinute = [NSString stringWithFormat:@"%d",sleepByte[4]];
                    NSString *endSecond = [NSString stringWithFormat:@"%d",sleepByte[5]];
                    
                    [sleepDict setObject:@(startHour.integerValue) forKey:@"startHour"];
                    [sleepDict setObject:@(startMinute.integerValue) forKey:@"startMinute"];
                    [sleepDict setObject:@(startSecond.integerValue) forKey:@"startSecond"];
                    [sleepDict setObject:@(endHour.integerValue) forKey:@"endHour"];
                    [sleepDict setObject:@(endMinute.integerValue) forKey:@"endMinute"];
                    [sleepDict setObject:@(endSecond.integerValue) forKey:@"endSecond"];
                    
                    [mutableDict setObject:sleepDict forKey:@"sleep"];
                    
                    //夜间模式 5bytes //100
                    
                    
                    //体温监测 6bytes
                    Byte *tempByte = (Byte *)[[data subdataWithRange:NSMakeRange(100, 6)] bytes];
                    
                    NSMutableDictionary *tempDict = [[NSMutableDictionary alloc] init];
                    NSString *switchTemp = [NSString stringWithFormat:@"%d",tempByte[0]];
                    NSString *startHourTemp = [NSString stringWithFormat:@"%d",tempByte[1]];
                    NSString *startMinuteTemp = [NSString stringWithFormat:@"%d",tempByte[2]];
                    NSString *endHourTemp = [NSString stringWithFormat:@"%d",tempByte[3]];
                    NSString *endMinuteTemp = [NSString stringWithFormat:@"%d",tempByte[4]];
                    NSString *interval = [NSString stringWithFormat:@"%d",tempByte[5]];
                    
                    [tempDict setObject:@(switchTemp.integerValue) forKey:@"switchTemp"];
                    [tempDict setObject:@(startHourTemp.integerValue) forKey:@"startHour"];
                    [tempDict setObject:@(startMinuteTemp.integerValue) forKey:@"startMinute"];
                    [tempDict setObject:@(endHourTemp.integerValue) forKey:@"endHour"];
                    [tempDict setObject:@(endMinuteTemp.integerValue) forKey:@"endMinute"];
                    [tempDict setObject:@(interval.integerValue) forKey:@"interval"];
                    
                }
                [mutableDict setObject:@(cmd) forKey:@"cmd"];
            }
        }
            break;
            
        case ECell_BLECMD_KEY_camera:
        {
            Byte *buffer = (Byte *)[data bytes];
            NSInteger state = buffer[13];
            [mutableDict setObject:@(state) forKey:@"state"];
            [mutableDict setObject:@(cmd) forKey:@"cmd"];
        }
            break;
            
        case ECell_BLECMD_KEY_realStep:
        case ECell_BLECMD_KEY_realHeartRate:
        case ECell_BLECMD_KEY_realBloodPressure:
        case ECell_BLECMD_KEY_realBloodOxygen:
        {
            mutableDict = [self parseRealTimeData:data cmdKey:cmd];
        }
            break;
        case 0x21: /// 透传数据
        {
            [data replaceBytesInRange:NSMakeRange(0, 13) withBytes:NULL length:0];
            [mutableDict setObject:data forKey:@"data"];
            [mutableDict setObject:@(cmd) forKey:@"cmd"];
            if (ECellBluetoothManager.shareInstance.result)
            {
                ECellBluetoothManager.shareInstance.result(ECell_BLE_COMMAND_SUCCESS_TYPE, mutableDict);
            }
           // [mutableDict removeObjectForKey:@"cmd"];
        }
            break;
        default:
            break;
    }
    if (cmd == 0x18)
        NSLog(@"-->0x18");
    if (cmd == 18)
        NSLog(@"--->0x18");
    
    return mutableDict;
}

//将十进制转化为十六进制
+ (NSString *)ToHex:(uint16_t)tmpid {
    NSString *nLetterValue;
    NSString *str =@"";
    uint16_t ttmpig;
    for (int i = 0; i<9; i++) {
        ttmpig=tmpid%16;
        tmpid=tmpid/16;
        switch (ttmpig)
        {
            case 10:
                nLetterValue =@"A";break;
            case 11:
                nLetterValue =@"B";break;
            case 12:
                nLetterValue =@"C";break;
            case 13:
                nLetterValue =@"D";break;
            case 14:
                nLetterValue =@"E";break;
            case 15:
                nLetterValue =@"F";break;

            default:
                nLetterValue = [NSString stringWithFormat:@"%u",ttmpig];
        }

        str = [nLetterValue stringByAppendingString:str];
        if (tmpid == 0) {
            break;
        }
    }

    return str;

}


// 将十六进制字符串转换成NSData
+ (NSMutableData *)convertHexStrToData:(NSString *)str {
    
    if (!str || [str length] == 0) {
        return nil;
    }
    
    NSMutableData *hexData = [[NSMutableData alloc] initWithCapacity:8];
    NSRange range;
    if ([str length] % 2 == 0) {
        range = NSMakeRange(0, 2);
    } else {
        range = NSMakeRange(0, 1);
    }
    for (NSInteger i = range.location; i < [str length]; i += 2) {
        unsigned int anInt;
        NSString *hexCharStr = [str substringWithRange:range];
        NSScanner *scanner = [[NSScanner alloc] initWithString:hexCharStr];
        
        [scanner scanHexInt:&anInt];
        NSData *entity = [[NSData alloc] initWithBytes:&anInt length:1];
        [hexData appendData:entity];
        
        range.location += range.length;
        range.length = 2;
    }
    
    return hexData;
}

// 将NSData转换成十六进制的字符串
+ (NSString *)convertDataToHexStr:(NSData *)data {
    
    if (!data || [data length] == 0) {
        return @"";
    }
    
    NSMutableString *string = [[NSMutableString alloc] initWithCapacity:[data length]];
    
    [data enumerateByteRangesUsingBlock:^(const void *bytes, NSRange byteRange, BOOL *stop) {
        unsigned char *dataBytes = (unsigned char*)bytes;
        for (NSInteger i = 0; i < byteRange.length; i++) {
            NSString *hexStr = [NSString stringWithFormat:@"%x", (dataBytes[i]) & 0xff];
            if ([hexStr length] == 2) {
                [string appendString:hexStr];
            } else {
                [string appendFormat:@"0%@", hexStr];
            }
        }
    }];
    
    return string;
}




/// 实时数据处理
/// @param data 数据
/// @param cmdKey 指令KEY
+ (NSMutableDictionary *)parseRealTimeData:(NSData *)data cmdKey:(ECell_BLECMD_KEY)cmdKey {
    NSMutableDictionary *mutableDict = [NSMutableDictionary dictionary];
    Byte *buffer = (Byte *)[data bytes];
    switch (cmdKey) {
        case ECell_BLECMD_KEY_realStep:
        {
            // 实时步数
            NSDateFormatter *formatter = [[NSDateFormatter alloc]init];
            formatter.dateFormat = @"yyyy-MM-dd-HH-mm";
            NSString *dateStr = [formatter stringFromDate:[NSDate date]];
            long long timestamp = [self changeTimeToMinutes:dateStr];
            NSString *steps = [[self class] intFromHexSrting:[NSString stringWithFormat:@"%02x%02x%02x%02x",buffer[13],buffer[14],buffer[15],buffer[16]]];
            Byte btValue[4];
            btValue[0] = buffer[20];
            btValue[1] = buffer[19];
            btValue[2] = buffer[18];
            btValue[3] = buffer[17];
            float result;
            result=*((float *)btValue);
            NSString *calories = [NSString stringWithFormat:@"%.1f",result];
            btValue[0] = buffer[24];
            btValue[1] = buffer[23];
            btValue[2] = buffer[22];
            btValue[3] = buffer[21];
            result=*((float *)btValue);
            [mutableDict setObject:@(timestamp) forKey:@"timestamp"];
            NSString *distance = [NSString stringWithFormat:@"%.0f",result*1000];
            [mutableDict setObject:@(steps.intValue) forKey:@"stepNumber"];
            [mutableDict setObject:@(calories.floatValue) forKey:@"stepCalorie"];
            [mutableDict setObject:@(distance.intValue) forKey:@"stepDistance"];
            [mutableDict setObject:@(cmdKey) forKey:@"cmd"];
        }
            break;
            
        case ECell_BLECMD_KEY_realHeartRate:
        {
            // 实时心率
            NSDateFormatter *formatter = [[NSDateFormatter alloc]init];
            formatter.dateFormat = @"yyyy-MM-dd-HH-mm";
            NSString *dateStr = [formatter stringFromDate:[NSDate date]];
            long long timestamp = [self changeTimeToMinutes:dateStr];
            NSString *heartRate = [NSString stringWithFormat:@"%02d",buffer[13]];
            [mutableDict setObject:@(timestamp) forKey:@"timestamp"];
            [mutableDict setObject:@(heartRate.intValue) forKey:@"heartRate"];
            //                    [mutableDict setObject:@(BleCMD_realHeartData) forKey:@"DataType"];
            [mutableDict setObject:@(cmdKey) forKey:@"cmd"];
        }
            break;
            
        case ECell_BLECMD_KEY_realBloodPressure:
        {
            // 实时血压
            Byte *buffer = (Byte *)[data bytes];
            long long time = (long long)[[NSDate date] timeIntervalSince1970];
            NSString *bloodpressureDate = [self nsdateToTime:time formatStr:@"yyyy-MM-dd"];
            NSInteger bloodpressureSystolic = buffer[13];
            NSInteger bloodpressureDiastolic = buffer[14];
            NSInteger bloodpressureAvg = (bloodpressureSystolic + 2 * bloodpressureDiastolic)/3;
            
            NSString *bloodpressureDetails = [NSString stringWithFormat:@"%ld|%ld|%lld",bloodpressureSystolic,bloodpressureDiastolic,time];
            
            [mutableDict setObject:@1 forKey:@"testingTimes"];
            [mutableDict setObject:@(bloodpressureSystolic) forKey:@"bloodpressureSystolic"];
            [mutableDict setObject:@(bloodpressureDiastolic) forKey:@"bloodpressureDiastolic"];
            [mutableDict setObject:@(bloodpressureAvg) forKey:@"bloodpressureAvg"];
            [mutableDict setObject:bloodpressureDetails forKey:@"bloodpressureDetails"];
            [mutableDict setObject:bloodpressureDate forKey:@"bloodpressureDate"];
            [mutableDict setObject:@(cmdKey) forKey:@"cmd"];
        }
            break;
            
        case ECell_BLECMD_KEY_realBloodOxygen:
        {
            // 实时血氧
            Byte *buffer = (Byte *)[data bytes];
            long long time = (long long)[[NSDate date] timeIntervalSince1970];
            NSString *bloodoxygen_times = [self nsdateToTime:time formatStr:@"yyyy-MM-dd"];
            NSString *saturation = [NSString stringWithFormat:@"%02d",buffer[13]];
            NSString *bloodoxygenDetails = [NSString stringWithFormat:@"%@|%lld",saturation,time];
            [mutableDict setObject:@(1) forKey:@"testingTime"];
            [mutableDict setObject:saturation forKey:@"saturation"];
            [mutableDict setObject:bloodoxygen_times forKey:@"bloodoxygenDate"];
            [mutableDict setObject:bloodoxygenDetails forKey:@"bloodoxygenDetails"];
            [mutableDict setObject:@(cmdKey) forKey:@"cmd"];
        }
            break;
            
        default:
            break;
    }
    return mutableDict;
}

/**
 将手环健康数据解析
 */
+ (NSMutableDictionary*)p_parseBraceletData_syncHealthWith:(NSData *)data {
    NSMutableDictionary *mutableDict = [NSMutableDictionary dictionary];
    Byte *bytes = (Byte *)[data bytes];
    Byte key = bytes[10];
    
    if (key == ECell_BLECMD_KEY_step_return) {
        // 计步数据
        NSArray *dataArr = [[self class] sepratorData:data];
        NSMutableData *completeData = [NSMutableData dataWithData:dataArr.firstObject];
        if (dataArr.count == 2) {
            NSData *lastData = dataArr.lastObject;
            [completeData appendData:[lastData subdataWithRange:NSMakeRange(13, lastData.length - 13)]];
        }
        NSMutableDictionary *stepDict = [[NSMutableDictionary alloc] init];
        NSInteger step_number = 0;
        CGFloat step_distance = 0.0;
        CGFloat step_calorie = 0.0;
        NSInteger step_duration = 0;
        NSMutableArray *stepDetailsArray = [[NSMutableArray alloc] init];
        NSString *year = [NSString stringWithFormat:@"%ld",[NSString stringWithFormat:@"%02d",bytes[13]].integerValue + 2000];
        NSString *month = [NSString stringWithFormat:@"%02d",bytes[14]];
        NSString *day = [NSString stringWithFormat:@"%02d",bytes[15]];
        NSString *step_times = [NSString stringWithFormat:@"%@-%@-%@",year,month,day];
        // 分段数据处理
        NSInteger count = (completeData.length - 17) / 12;
        NSString *tempSteps = @"0";
        NSString *tempCalorie= @"0";
        NSString *tempDistance = @"0";
        for (int i = 0; i < count; i ++) {
            NSData *stepData = [completeData subdataWithRange:NSMakeRange(17+i*12, 12)];
            NSString *steps = [[self class] HexToUnsignedInt:stepData dataRange:NSMakeRange(0, 4)];
            NSString *calorie = [[self class] HexToUnsignedInt:stepData dataRange:NSMakeRange(4, 4)];
            NSString *distance = [[self class] HexToUnsignedInt:stepData dataRange:NSMakeRange(8, 4)];
            // 记录该时段的步数（分段数据：step|cal|km|时段,）
            if (steps.integerValue < tempSteps.integerValue) {
                steps = tempSteps;
            }
            NSString *currenSteps = [NSString stringWithFormat:@"%ld",steps.integerValue - tempSteps.integerValue];
            tempSteps = steps;
            
            if (calorie.integerValue < tempCalorie.integerValue) {
                calorie = tempCalorie;
            }
            NSString *currenCalorie = [NSString stringWithFormat:@"%ld",calorie.integerValue - tempCalorie.integerValue];
            tempCalorie = calorie;
            
            if (distance.integerValue < tempDistance.integerValue) {
                distance = tempDistance;
            }
            NSString *currenDistance = [NSString stringWithFormat:@"%ld",distance.integerValue - tempDistance.integerValue];
            tempDistance = distance;
            // 计步详情
            NSString *stepDetails = [NSString stringWithFormat:@"%@|%@|%@|%d",currenSteps, currenCalorie, currenDistance, i];
            [stepDetailsArray addObject:stepDetails];
            // 当前步数 || 计步卡路里 || 计步距离 || 计步时长（1km==15分钟,四舍五入）
            if (i == count - 1) {
                step_number = steps.integerValue;
                step_calorie = [calorie doubleValue];
                step_distance = [distance doubleValue];
                step_duration = (step_distance / 1000.0) * 15 + 0.5;
            }
        }
        [stepDict setObject:@(step_number) forKey:@"stepNumber"];
        [stepDict setObject:@(step_distance)forKey:@"stepDistance"];
        [stepDict setObject:@(step_calorie) forKey:@"stepCalorie"];
        [stepDict setObject:@(step_duration) forKey:@"stepDuration"];
        [stepDict setObject:[stepDetailsArray componentsJoinedByString:@","] forKey:@"stepDetails"];
        [stepDict setObject:step_times forKey:@"dataDate"];
        if (stepDict) {
            [mutableDict setObject:stepDict forKey:@"step"];
        }
        
    }else if (key == ECell_BLECMD_KEY_sleep_return) {
        // 睡眠数据拼接
        NSMutableDictionary *sleepdict = [[NSMutableDictionary alloc] init];
        NSInteger deepsleep_duration = 0;
        NSInteger lightsleep_duration = 0;
        NSInteger awake_duration = 0;
        NSInteger awakeNumber = 0;
        NSString *fallingsleep_times = [[NSString alloc] init];
        NSString *awake_times = [[NSString alloc] init];
        NSString *sleep_times = [[NSString alloc] init];
        // 睡眠详情数组
        NSMutableArray *sleepDetailArr = [[NSMutableArray alloc]init];
        // 睡眠数据处理
        NSData *contonData = [data subdataWithRange:NSMakeRange(13, data.length - 13)];
        Byte *buffer = (Byte *)[contonData bytes];
        NSString *year = [NSString stringWithFormat:@"%ld",[NSString stringWithFormat:@"%02d",buffer[0]].integerValue + 2000];
        NSString *month = [NSString stringWithFormat:@"%02d",buffer[1]];
        NSString *day = [NSString stringWithFormat:@"%02d",buffer[2]];
        sleep_times = [NSString stringWithFormat:@"%@-%@-%@",year,month,day];
        // 睡眠数据个数
        NSInteger count = contonData.length / 3 - 1;
        for (int i = 0; i < count; i ++) {
            // 取出第一条睡眠数据,第二条睡眠数据
            NSData *sleepData = [contonData subdataWithRange:NSMakeRange(3 + i * 3, 3)];
            NSData *nextData = [[NSData alloc] init];
            if (i < count - 1) {
                nextData = [contonData subdataWithRange:NSMakeRange(3 + ((i + 1)* 3), 3)];
            }else {
                // 最后一条数据或仅一条数据不处理
                break;
            }
            // 获取开始小时,结束时间
            NSString *startHour = [NSString stringWithFormat:@"%02d",((Byte*)([sleepData bytes]))[1]];
            NSString *startMinute = [NSString stringWithFormat:@"%02d",((Byte*)([sleepData bytes]))[2]];
            NSString *endHour = [NSString stringWithFormat:@"%02d",((Byte*)([nextData bytes]))[1]];
            NSString *endMinute = [NSString stringWithFormat:@"%02d",((Byte*)([nextData bytes]))[2]];
            
            if (i == 0) {
                // 首条开始时间若是在09：00 - 21：00之内，则设置为21：00
                if (startHour.integerValue < 21) {
                    if (startHour.integerValue > 9) {
                        startHour = @"21";
                        startMinute = @"00";
                    }else {
                        // 若首条数据小时为小于9点,则数据日期+1天
                        NSString *time = [[self class] getTimestampWithTime:[NSString stringWithFormat:@"%@-%@-%@",year,month,day]];
                        NSArray *timeArr = [[self nsdateToTime:[time integerValue] + 86400 formatStr:@"yyyy-MM-dd"] componentsSeparatedByString:@"-"];
                        year = timeArr[0];
                        month = timeArr[1];
                        day = timeArr[2];
                    }
                }
                // 记录入睡时间
                fallingsleep_times = [NSString stringWithFormat:@"%@-%@-%@ %@:%@",year,month,day,startHour,startMinute];
            }
            // 当前时段开始时间
            NSString *startSleepTime = [NSString stringWithFormat:@"%@-%@-%@-%@-%@",year,month,day,startHour,startMinute];
            
            // 处理跨天的数据
            if ([endHour integerValue] < [startHour integerValue]) {
                NSString *endTime = [[self class] getTimestampWithTime:[NSString stringWithFormat:@"%@-%@-%@-%@-%@",year,month,day,endHour,endMinute]];
                NSArray *timeArr = [[self nsdateToTime:[endTime integerValue] + 86400 formatStr:@"yyyy-MM-dd-HH-mm"] componentsSeparatedByString:@"-"];
                // 重新赋值年月日
                year = timeArr[0];
                month = timeArr[1];
                day = timeArr[2];
            }
            // 当前时段结束时间
            NSString *endSleepTime = [NSString stringWithFormat:@"%@-%@-%@-%@-%@",year,month,day,endHour,endMinute];
            if (i == count - 2) {
                // 最后的结束时间处理（结束时间大于9：00，默认9：00）
                if (endHour.integerValue > 9 || (endHour.integerValue >=9 && endMinute.integerValue > 0)) {
                    endSleepTime = [NSString stringWithFormat:@"%@-%@-%@-09-00",year,month,day];
                }
                // 记录睡来时间
                NSArray *endtimeArr = [endSleepTime componentsSeparatedByString:@"-"];
                awake_times = [NSString stringWithFormat:@"%@-%@-%@ %@:%@",endtimeArr[0],endtimeArr[1],endtimeArr[2],endtimeArr[3],endtimeArr[4]];
            }
            
            // 睡眠类型
            NSInteger sleepType = ((Byte*)([sleepData bytes]))[0];
            // 睡眠详情数据(睡眠类型|开始时间戳|持续时长|结束时间戳,..)
            long long startSleep = [self changeTimeToMinutes:startSleepTime];
            long long endSleep = [self changeTimeToMinutes:endSleepTime];
            // 睡眠持续时间
            NSInteger sleepTime = endSleep - startSleep;
            if (sleepTime <= 0) {
                continue;
            }
            sleepTime = sleepTime / 60;
            NSString *sleepDetail = [NSString stringWithFormat:@"%ld|%lld|%ld|%lld",sleepType,startSleep,sleepTime,endSleep];
            [sleepDetailArr addObject:sleepDetail];
            // 记录睡来时间
            NSArray *endtimeArr = [endSleepTime componentsSeparatedByString:@"-"];
            awake_times = [NSString stringWithFormat:@"%@-%@-%@ %@:%@",endtimeArr[0],endtimeArr[1],endtimeArr[2],endtimeArr[3],endtimeArr[4]];
            switch (sleepType) {
                case 0:
                    awake_duration += sleepTime;
                    awakeNumber += 1;
                    break;
                case 1:
                    lightsleep_duration += sleepTime;
                    break;
                case 2:
                    deepsleep_duration += sleepTime;
                    break;
                    
                default:
                    break;
            }
        }
        NSInteger totalSleepDuration = deepsleep_duration + lightsleep_duration;
        if (totalSleepDuration > 0) {   //总睡眠时长大于0的睡眠才有效
            [sleepdict setObject:@(deepsleep_duration) forKey:@"deepSleepDuration"];
            [sleepdict setObject:@(lightsleep_duration) forKey:@"lightSleepDuration"];
            [sleepdict setObject:@(awake_duration) forKey:@"awakeDuration"];
            [sleepdict setObject:@(awakeNumber) forKey:@"awakeNumber"];
            [sleepdict setObject:@(totalSleepDuration) forKey:@"totalSleepDuration"];
            NSString *sleepDetailStr = [sleepDetailArr componentsJoinedByString:@","];
            [sleepdict setObject:sleepDetailStr forKey:@"sleepDetails"];
            [sleepdict setObject:fallingsleep_times forKey:@"fallingSleepTimes"];
            [sleepdict setObject:awake_times forKey:@"awakeTimes"];
            [sleepdict setObject:sleep_times forKey:@"dataDate"];
            [mutableDict setObject:sleepdict forKey:@"sleep"];
        }
    }else if(key == ECell_BLECMD_KEY_heartRate_return){
        // 心率数据
        NSMutableDictionary *heartRateDict = [[NSMutableDictionary alloc] init];
        NSArray *dataArr = [[self class] sepratorData:data];
        // 合并一天完整心率数据
        NSMutableData *completeData = [NSMutableData dataWithData:dataArr.firstObject];
        for (NSInteger i = 1; i < dataArr.count; i ++) {
            NSData *hrData = dataArr[i];
            [completeData appendData:[hrData subdataWithRange:NSMakeRange(13, hrData.length - 13)]];
        }
        // 心率日期
        NSString *year = [NSString stringWithFormat:@"%ld",[NSString stringWithFormat:@"%02d",bytes[13]].integerValue + 2000];
        NSString *month = [NSString stringWithFormat:@"%02d",bytes[14]];
        NSString *day = [NSString stringWithFormat:@"%02d",bytes[15]];
        NSString *heartDate = [NSString stringWithFormat:@"%@-%@-%@",year,month,day];
        NSMutableString *heartDetails = [[NSMutableString alloc] init];
        NSInteger heartMax = 0;
        NSInteger heartMin = 0;
        NSInteger heartAvg = 0;
        NSInteger tampTotalHeart = 0;
        // 手环心率详情数据拼接
        NSInteger count = (completeData.length - 13) / 7;
        for (int i = 0; i < count; i ++) {
            NSData *contonData = [completeData subdataWithRange:NSMakeRange(13 + i * 7, 7)];
            Byte *buffer = (Byte *)[contonData bytes];
            NSString *hour = [NSString stringWithFormat:@"%02d",buffer[3]];
            NSString *minute = [NSString stringWithFormat:@"%02d",buffer[4]];
            NSString *second = [NSString stringWithFormat:@"%02d",buffer[5]];
            NSString *heartRate = [NSString stringWithFormat:@"%02d",buffer[6]];
            //（心率值|时间戳,..)
            long long timeStamp = [self changeTimeToMinutes:[NSString stringWithFormat:@"%@-%@-%@-%@-%@-%@",year,month,day,hour,minute,second]];
            [heartDetails appendString:[NSString stringWithFormat:@"%@|%lld",heartRate,timeStamp]];
            if (i < count - 1) {
                [heartDetails appendString:@","];
            }
            if (heartRate.integerValue > heartMax) {
                heartMax = heartRate.integerValue;
            }
            if (heartRate.integerValue < heartMin || heartMin == 0) {
                heartMin = heartRate.integerValue;
            }
            tampTotalHeart += heartRate.integerValue;
        }
        heartAvg = tampTotalHeart / count;
        [heartRateDict setObject:heartDate forKey:@"dataDate"];
        [heartRateDict setObject:@(heartMax) forKey:@"heartMax"];
        [heartRateDict setObject:@(heartMin) forKey:@"heartMin"];
        [heartRateDict setObject:@(heartAvg) forKey:@"heartAvg"];
        [heartRateDict setObject:heartDetails forKey:@"heartDetails"];
        [mutableDict setObject:heartRateDict forKey:@"heartRate"];
    }else if (key == ECell_BLECMD_KEY_bloodPressure_return) {
        // 手环血压
        Byte *buffer = (Byte *)[data bytes];
        
        NSString *year = [NSString stringWithFormat:@"%02d",buffer[13]];
        NSString *month = [NSString stringWithFormat:@"%02d",buffer[14]];
        NSString *day = [NSString stringWithFormat:@"%02d",buffer[15]];
        NSString *hour = [NSString stringWithFormat:@"%02d",buffer[16]];
        NSString *minute = [NSString stringWithFormat:@"%02d",buffer[17]];
        NSString *second = [NSString stringWithFormat:@"%02d",buffer[18]];
        NSString *shrinkage = [NSString stringWithFormat:@"%02d",buffer[19]];
        NSString *diastolic = [NSString stringWithFormat:@"%02d",buffer[20]];
        
        [mutableDict setObject:year forKey:@"Year"];
        [mutableDict setObject:month forKey:@"Month"];
        [mutableDict setObject:day forKey:@"Day"];
        [mutableDict setObject:hour forKey:@"Hour"];
        [mutableDict setObject:minute forKey:@"Minute"];
        [mutableDict setObject:second forKey:@"Second"];
        [mutableDict setObject:shrinkage forKey:@"shrinkage"];
        [mutableDict setObject:diastolic forKey:@"diastolic"];
        
    }
    else if (key == ECell_BLECMD_KEY_bloodOxygen_return)
    {
        // 血氧数据
        NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
        NSArray *dataArr = [[self class] sepratorData:data];
        // 合并一天完整数据
        NSMutableData *completeData = [NSMutableData dataWithData:dataArr.firstObject];
        for (NSInteger i = 1; i < dataArr.count; i ++) {
            NSData *itemData = dataArr[i];
            [completeData appendData:[itemData subdataWithRange:NSMakeRange(13, itemData.length - 13)]];
        }
        // 日期
        NSString *year = [NSString stringWithFormat:@"%ld",[NSString stringWithFormat:@"%02d",bytes[13]].integerValue + 2000];
        NSString *month = [NSString stringWithFormat:@"%02d",bytes[14]];
        NSString *day = [NSString stringWithFormat:@"%02d",bytes[15]];
        NSString *dataDate = [NSString stringWithFormat:@"%@-%@-%@",year,month,day];
        NSMutableString *details = [[NSMutableString alloc] init];
        NSInteger bloodOxygenMax = 0;
        NSInteger bloodOxygenMin = 0;
        NSInteger bloodOxygenAvg = 0;
        NSInteger tempTotal = 0;
        // 详情数据拼接
        NSInteger count = (completeData.length - 13) / 7;
        for (int i = 0; i < count; i ++)
        {
            NSData *contonData = [completeData subdataWithRange:NSMakeRange(13 + i * 7, 7)];
            Byte *buffer = (Byte *)[contonData bytes];
            NSString *hour = [NSString stringWithFormat:@"%02d",buffer[3]];
            NSString *minute = [NSString stringWithFormat:@"%02d",buffer[4]];
            NSString *second = [NSString stringWithFormat:@"%02d",buffer[5]];
            NSString *value = [NSString stringWithFormat:@"%02d",buffer[6]];
            //（血氧值|时间戳,..)
            long long timeStamp = [self changeTimeToMinutes:[NSString stringWithFormat:@"%@-%@-%@-%@-%@-%@",year,month,day,hour,minute,second]];
            [details appendString:[NSString stringWithFormat:@"%@|%lld",value,timeStamp]];
            if (i < count - 1) {
                [details appendString:@","];
            }
            if (value.integerValue > bloodOxygenMax) {
                bloodOxygenMax = value.integerValue;
            }
            if (value.integerValue < bloodOxygenMin || bloodOxygenMin == 0) {
                bloodOxygenMin = value.integerValue;
            }
            tempTotal += value.integerValue;
        }
        bloodOxygenAvg = tempTotal / count;
        [dict setObject:dataDate forKey:@"dataDate"];
        [dict setObject:@(bloodOxygenMax) forKey:@"bloodOxygenMax"];
        [dict setObject:@(bloodOxygenMin) forKey:@"bloodOxygenMin"];
        [dict setObject:@(bloodOxygenAvg) forKey:@"bloodOxygenAvg"];
        [dict setObject:details forKey:@"details"];
        [mutableDict setObject:dict forKey:@"BLOODOXYGEN"];
        
    } else if (key == ECell_BLECMD_KEY_sport_return) {
        //  1.分割手环GPS运动模式数据处理
        NSMutableDictionary *sportDict = [[NSMutableDictionary alloc] init];
        NSArray *dataArr = [[self class] sepratorData:data];
        // 合并完整运动数据
        NSMutableData *completeData = [NSMutableData dataWithData:dataArr.firstObject];
        for (NSInteger i = 1; i < dataArr.count; i ++) {
            NSData *sportData = dataArr[i];
            [completeData appendData:[sportData subdataWithRange:NSMakeRange(13, sportData.length - 13)]];
        }
        NSData *sportData = [completeData subdataWithRange:NSMakeRange(13, completeData.length - 13)];
        Byte *buffer = (Byte *)[sportData bytes];
        // 前58字节
        NSInteger sportKey = [[NSString stringWithFormat:@"%02d",buffer[0]] integerValue];
        if (sportKey == KEY_SPORTMODEL_SPORT) {
            NSInteger version = [[NSString stringWithFormat:@"%02d",buffer[1]] integerValue];
            NSInteger sportType = [[NSString stringWithFormat:@"%02d",buffer[2]] integerValue];
            NSString *year = [NSString stringWithFormat:@"%ld",[NSString stringWithFormat:@"%02d",buffer[3]].integerValue + 2000];
            NSString *month = [NSString stringWithFormat:@"%02d",buffer[4]];
            NSString *day = [NSString stringWithFormat:@"%02d",buffer[5]];
            NSString *hour = [NSString stringWithFormat:@"%02d",buffer[6]];
            NSString *minute = [NSString stringWithFormat:@"%02d",buffer[7]];
            NSString *second = [NSString stringWithFormat:@"%02d",buffer[8]];
            NSString *dataDate = [NSString stringWithFormat:@"%@-%@-%@ %@:%@:%@",year,month,day,hour,minute,second];
            NSInteger duration = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(9, 4)] integerValue];
            NSInteger distance = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(13, 4)] integerValue];
            NSInteger calorie = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(17, 4)] integerValue];
            NSInteger stepNumber = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(21, 4)] integerValue];
            NSInteger heartMax = [[NSString stringWithFormat:@"%02d",buffer[25]] integerValue];
            NSInteger heartMin = [[NSString stringWithFormat:@"%02d",buffer[26]] integerValue];
            NSInteger heartAvg = [[NSString stringWithFormat:@"%02d",buffer[27]] integerValue];
            NSInteger stepFrequencyMax = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(28, 2)] integerValue];
            NSInteger stepFrequencyAvg = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(30, 2)] integerValue];
            NSInteger stepFrequencyMin = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(32, 2)] integerValue];
            NSInteger paceMax = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(34, 2)] integerValue];
            NSInteger paceAvg = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(36, 2)] integerValue];
            NSInteger paceMin = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(38, 2)] integerValue];
            NSInteger startAltitude = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(40, 2)] integerValue];
            NSInteger endAltitude = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(42, 2)] integerValue];
            NSInteger gpsCount = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(44, 4)] integerValue];
            NSInteger kmCount = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(48, 2)] integerValue];
            NSInteger stepCount = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(50, 2)] integerValue];
            NSInteger heartCount = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(52, 4)] integerValue];
            NSInteger trainDetailCount = [[[self class]HexToUnsignedInt:sportData dataRange:NSMakeRange(56, 2)] integerValue];
            [sportDict setObject:@(version) forKey:@"version"];
            [sportDict setObject:@(sportType) forKey:@"sportType"];
            [sportDict setObject:dataDate forKey:@"dataDate"];
            [sportDict setObject:@(duration) forKey:@"duration"];
            [sportDict setObject:@(distance) forKey:@"distance"];
            CGFloat calorieKcal = calorie / 1000.0;
            [sportDict setObject:@(calorieKcal) forKey:@"calorie"];
            [sportDict setObject:@(stepNumber) forKey:@"stepNumber"];
            [sportDict setObject:@(heartMax) forKey:@"heartMax"];
            [sportDict setObject:@(heartMin) forKey:@"heartMin"];
            [sportDict setObject:@(heartAvg) forKey:@"heartAvg"];
            [sportDict setObject:@(stepFrequencyMax) forKey:@"stepFrequencyMax"];
            [sportDict setObject:@(stepFrequencyAvg) forKey:@"stepFrequencyAvg"];
            [sportDict setObject:@(stepFrequencyMin) forKey:@"stepFrequencyMin"];
            [sportDict setObject:@(paceMax) forKey:@"paceMax"];
            [sportDict setObject:@(paceAvg) forKey:@"paceAvg"];
            [sportDict setObject:@(paceMin) forKey:@"paceMin"];
            [sportDict setObject:@(startAltitude) forKey:@"startAltitude"];
            [sportDict setObject:@(endAltitude) forKey:@"endAltitude"];
            [sportDict setObject:@(gpsCount) forKey:@"gpsCount"];
            [sportDict setObject:@(kmCount) forKey:@"kmCount"];
            [sportDict setObject:@(stepCount) forKey:@"stepCount"];
            [sportDict setObject:@(heartCount) forKey:@"heartCount"];
            [sportDict setObject:@(trainDetailCount) forKey:@"trainDetailCount"];
        }
        [mutableDict setObject:sportDict forKey:@"sport"];
        [mutableDict setObject:@(key) forKey:@"cmd"];
        // 其他分组数据
//        #define KEY_SPORTMODEL_GPS 0xB0
//        #define KEY_SPORTMODEL_KM 0xB1
//        #define KEY_SPORTMODEL_STEP 0xB2
//        #define KEY_SPORTMODEL_HRS 0xB3
//        #define KEY_SPORTMODEL_SWIM 0xB4
    }
    return mutableDict;
}

/**
 分割数据
 
 @param data 数据源
 @return 包含分割后data的数组
 */
+ (NSArray *)sepratorData:(NSData *)data {
    
    NSMutableArray *mutableArr = [[NSMutableArray alloc]init];
    NSData *seprator = [DataSeparator dataUsingEncoding:NSUTF8StringEncoding];
    // 新建索引优化分割数据流的性能 modify by fangtao 2018.12.01
    const int nSeparatorSize = 4;
    NSUInteger currentLocation = 0;      // 当前遍历到的字节流的位置
    
    while (currentLocation < data.length) {
        // 在copyData中查找是否有"!@#$" 分隔字节流
        NSRange sepratorRange = [data rangeOfData:seprator options:0 range:NSMakeRange(currentLocation, data.length - currentLocation)];
        NSData *subData = nil;
        if (sepratorRange.length == nSeparatorSize) {
            subData = [data subdataWithRange:NSMakeRange(currentLocation, sepratorRange.location - currentLocation)];
            // 变更循环条件
            currentLocation = sepratorRange.location + nSeparatorSize;
        } else {
            subData = [data subdataWithRange:NSMakeRange(currentLocation, data.length - currentLocation)];
            // 变更循环条件
            currentLocation = data.length;
        }
        
        // 子字节流长度大于0 才加入到字节流数组中
        if (subData.length > 0) {
            [mutableArr addObject:subData];
        }
    }
    return mutableArr;
}

/**
 JSON格式字符串转数组或字典
 
 @param JSONString 字符串
 @return 数组或字典
 */
+ (id)parseJSONStringToNSDictionary:(NSString *)JSONString {
    NSData *JSONData = [JSONString dataUsingEncoding:NSUTF8StringEncoding];
    NSError *err;
    id object = [NSJSONSerialization JSONObjectWithData:JSONData options:NSJSONReadingMutableLeaves error:&err];
    if (err) {
        NSLog(@"错误：不是JSON格式的字符串");
        return nil;
    }
    return object;
}

/**
 16进账转单精度
 
 @param data 16进制数据
 @param range 范围
 @return 单精度
 */
+ (NSString *)HexToFloat:(NSData *)data dataRange:(NSRange)range {
    NSData *tempData = [data subdataWithRange:range];
    float f = *((float *)tempData.bytes);
    return [NSString stringWithFormat:@"%.3f",f];
}

/**
 16进制转双精度
 
 @param data 16进制数据
 @param range 范围
 @return 双精度
 */
+ (NSString *)HexToDouble:(NSData *)data dataRange:(NSRange)range {
    NSData *tempData = [data subdataWithRange:range];
    double df = *((double *)tempData.bytes);
    return [NSString stringWithFormat:@"%lf",df];
}

/**
 *  16进制转10进制数字(手环经纬度坐标转换使用)
 *
 *  @param data data
 *
 *  @return 十进制数据
 */
+ (double)HexToCoordinates:(NSData *)data dataRange:(NSRange)range {
    NSData *tempData = [data subdataWithRange:range];
    NSString *tempDataStr = [tempData.description substringWithRange:NSMakeRange(1, tempData.length *2 )];
    double decimalism= (int)strtoll([tempDataStr UTF8String],0,16) / pow(10, 6);
    return decimalism;
}

/**
 *  16进制转10进制数字（有符号）
 *
 *  @param data data
 *
 *  @return 十进制数据
 */
+ (NSString *)HexToInt:(NSData *)data dataRange:(NSRange)range {
    NSData *tempData = [data subdataWithRange:range];
    Byte *bytes = (Byte *)[tempData bytes];
    int len = (int)tempData.length;
    int res = bytesToInt(bytes, len);
    if (len == 2) {
        return [NSString stringWithFormat:@"%d", (short)res];
    }
    return [NSString stringWithFormat:@"%d", res];
}

/**
 *  16进制转10进制数字 (无符号)
 *
 *  @param data data
 *
 *  @return 十进制数据
 */
+ (NSString *)HexToUnsignedInt:(NSData *)data dataRange:(NSRange)range {
    NSData *tempData = [data subdataWithRange:range];
    Byte *bytes = (Byte *)[tempData bytes];
    return [NSString stringWithFormat:@"%u", bytesToInt(bytes, (int)tempData.length)];
}

/**
 byte数组转成无符号整数 add by fangtao 2018.12.01
 
 @param src byte数组
 @param nSize 数组长度
 @return unsigned数值
 */
int bytesToInt(Byte *src, int nSize)
{
    int value = 0;
    for (int i = 0; i<nSize; i++) {
        value |= (src[i] & 0xff) << (8 * (nSize - i - 1));
    }
    return value;
}


/**
 *  16进制转10进制数字 (无符号)(小端模式)
 *
 *  @param data data
 *
 *  @return 十进制数据
 */
+ (NSString *)HexToUnsignedIntLittle:(NSData *)data dataRange:(NSRange)range {
    NSData *tempData = [data subdataWithRange:range];
    Byte *bytes = (Byte *)[tempData bytes];
    return [NSString stringWithFormat:@"%u", bytesToIntLittle(bytes, (int)tempData.length)];
}

/**
 byte数组转成无符号整数 add by fangtao 2018.12.01 (小端模式)
 
 @param src byte数组
 @param nSize 数组长度
 @return unsigned数值
 */
int bytesToIntLittle(Byte *src, int nSize)
{
    int value = 0;
    for (int i = 0; i<nSize; i++) {
        value |= (src[i] & 0xff) << (8 * i);
    }
    return value;
}

/**
 *  16进制字符串转10进制数字
 *
 *  @param hexString 16进制字符串
 *
 *  @return 十进制数据
 */
+ (NSString *)intFromHexSrting:(NSString *)hexString {
    NSString * temp10 = [NSString stringWithFormat:@"%lu",strtoul([hexString UTF8String],0,16)];
    
    return temp10;
}

/**
 将周期int类型转成位字符串（十进制转二进制字符串）
 */
+ (NSString *)cycleIntToString:(int)cycle {
    
    NSMutableString *cycleStr = [[NSMutableString alloc] init];
    for (int i = 0; i < 7; i ++ ) {
        
        int temp = cycle;
        temp = temp & (1<<i);
        temp = temp >> i;
        [cycleStr appendString:[NSString stringWithFormat:@"%d",temp]];
    }
    return cycleStr;
}

/**
 十进制转二进制字符串(逆序)
 */
+ (NSString *)cycleIntToReverseString:(int)cycle {
    int n = 7;
    int mask = 1<<(n-1);
    NSMutableString *cycleStr = [[NSMutableString alloc]init];
    for (int i = 1; i<=n; i++) {
        [cycleStr insertString:[NSString stringWithFormat:@"%d", (cycle & mask)==0?0:1] atIndex:0];
        cycle <<= 1;
    }
    return cycleStr;
}

/**
 获取时间戳字符串
 
 @param time 2017-01-01-01-01格式的时间
 
 @return 时间戳字符串
 */
+ (NSString *)getTimestampWithTime:(NSString *)time {
    
    if (!time.length) {
        return @"";
    }
    NSArray *arr = [time componentsSeparatedByString:@"-"];
    NSDateFormatter *inputFormatter = [[NSDateFormatter alloc] init];
    [inputFormatter setCalendar: [[NSCalendar alloc]
                                  initWithCalendarIdentifier:NSCalendarIdentifierGregorian]];
    [inputFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US"]];
    switch (arr.count) {
        case 3:
            [inputFormatter setDateFormat:@"yy-MM-dd"];
            break;
            
        case 4:
            [inputFormatter setDateFormat:@"yy-MM-dd-HH"];
            break;
            
        case 5:
            [inputFormatter setDateFormat:@"yy-MM-dd-HH-mm"];
            break;
            
            
        case 6:
            [inputFormatter setDateFormat:@"yy-MM-dd-HH-mm-ss"];
            break;
            
            
        default:
            break;
    }
    NSDate* inputDate = [inputFormatter dateFromString:time];
    NSString *timestamp = [NSString stringWithFormat:@"%ld",(long)[inputDate timeIntervalSince1970]];
    return timestamp;
}

//获取16进制字符串
+ (NSString *)data2HexString: (NSData *) data {
    NSString *string = data.description;
    string = [string stringByReplacingOccurrencesOfString:@"<" withString:@""];
    string = [string stringByReplacingOccurrencesOfString:@">" withString:@""];
    string = [string stringByReplacingOccurrencesOfString:@" " withString:@""];
    return string;
}

/**
 正序mac地址
 */
+ (NSString *)getPositiveSequenceMacWithData:(NSData *)data {
    NSString *dataStr = [self hexStringFromData:data];
    NSMutableString *mutableMac = [[NSMutableString alloc]init];;
    if (dataStr) {
        dataStr = [dataStr stringByReplacingOccurrencesOfString:@" " withString:@""];
        dataStr = [dataStr stringByReplacingOccurrencesOfString:@"<" withString:@""];
        dataStr = [dataStr stringByReplacingOccurrencesOfString:@">" withString:@""];
        for (int i = 0; i < 6; i++) {
            NSString *str = [dataStr substringWithRange:NSMakeRange(i*2, 2)];
            if (i == 5) {
                [mutableMac appendString:[NSString stringWithFormat:@"%@",str]];
            }else {
                [mutableMac appendString:[NSString stringWithFormat:@"%@:",str]];
            }
        }
    }
    return [mutableMac uppercaseString];
}

//将yyyy-MM-dd-HH-mm 格式时间转换成时间戳
+ (long)changeTimeToMinutes:(NSString *)timeStr{
    long time = 0;
    NSDateFormatter *format=[[NSDateFormatter alloc] init];
    [format setCalendar: [[NSCalendar alloc]
                          initWithCalendarIdentifier:NSCalendarIdentifierGregorian]];
    NSInteger count = [timeStr componentsSeparatedByString:@"-"].count;
    if (count == 5) {
        [format setDateFormat:@"yyyy-MM-dd-HH-mm"];
    }
    if (count == 6) {
        [format setDateFormat:@"yyyy-MM-dd-HH-mm-ss"];
    }
    NSDate *fromdate=[format dateFromString:timeStr];
    time= (long)[fromdate timeIntervalSince1970];
    //    HQLog(@"%ld",time);
    return time;
}

/**
 将时间戳按指定格式时间输出
 
 @param spString  秒
 @param formatStr 指定格式
 
 */
+ (NSString*)nsdateToTime:(long long)spString formatStr:(NSString *)formatStr
{
    NSDate *date = [NSDate dateWithTimeIntervalSince1970:spString];
    NSTimeZone *zone = [NSTimeZone systemTimeZone]; // 获得系统的时区
    NSTimeInterval time = [zone secondsFromGMTForDate:date];
    [date dateByAddingTimeInterval:time];
    NSDateFormatter *dateFormat=[[NSDateFormatter alloc]init];
    NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"zh"];
    [dateFormat setLocale:locale];
    [dateFormat setCalendar: [[NSCalendar alloc]
                              initWithCalendarIdentifier:NSCalendarIdentifierGregorian]];
    [dateFormat setDateFormat:formatStr];
    NSString* string=[dateFormat stringFromDate:date];
    
    return string;
}

/**
 Data转16进制字符串
 */
+ (NSString *)hexStringFromData:(NSData*)data {
    if (![data isKindOfClass:[NSData class]]) {
        return nil;
    }
    if (data == nil) {
        return nil;
    }
    NSMutableString *hexString = [NSMutableString string];
    const unsigned char *p = [data bytes];
    for (int i=0; i < [data length]; i++) {
        [hexString appendFormat:@"%02x", *p++];
    }
    return hexString;
}

@end
