//
//  ECellCommandProvider.m
//  ECellBluetooth_Example
//
//  Created by lemo. on 2019/5/14.
//  Copyright © 2019 chentingfeng. All rights reserved.
//

#import "ECellCommandProvider.h"
#import <ECellBluetooth/ECellBLEItem.h>
#import <ECellBluetooth/ECellBluetoothManager.h>

// 去除警告
#define PerformSelectorLeakWarning(Stuff) \
do { \
_Pragma("clang diagnostic push") \
_Pragma("clang diagnostic ignored \"-Warc-performSelector-leaks\"") \
Stuff; \
_Pragma("clang diagnostic pop") \
} while (0)

#define LOne_Version   0                    //L1层版本
#define LTwo_Version   0                    //L2层版本
#define LOne_Length    5                    //L2层无Value是的总长度
#define Identification 0xCB                 //识别标志

@implementation ECellCommandProvider

#pragma mark - Packet

/// 指令打包
/// @param cmd 指令类型
/// @param cmdKey 指令KEY类型
/// @param paramData 参数数据
/// @param result 回调结果
+ (void)packetData:(ECell_BLECMD)cmd
            cmdKey:(ECell_BLECMD_KEY)cmdKey
         ParamData:(nullable NSData *)paramData
       ResultBlock:(ResultBlock)result {
    // 序号
    NSInteger sid = [self getSendCmdID];
    // 1.CRC计算
    Byte cmdKeybyte[5];
    cmdKeybyte[0] = cmd;
    cmdKeybyte[1] = LTwo_Version;
    cmdKeybyte[2] = cmdKey;
    cmdKeybyte[3] = (Byte)(paramData.length >> 8);
    cmdKeybyte[4] = (Byte)(paramData.length & 0xff);
    // L2层数据构建
    NSMutableData *contentData = [NSMutableData new];
    [contentData appendBytes:cmdKeybyte length:5];
    [contentData appendData:paramData];
    // 8位CRC计算
    int crc = 0xff;
    for (int i = 0; i < contentData.length; i++) {
        crc^= ((Byte*)([contentData bytes]))[i] & 0xFF;
        for (int j = 0; j < 8; j++) {
            if((crc & 1) !=0){
                crc >>=1;
                crc^=0xB8;
            }else{
                crc>>=1;
            }
        }
    }
    //2.包头L1 （8bytes）
    Byte byte[8];
    byte[0] = Identification;//识别标志
    byte[1] = [self stitchWithErr:1 Ack:0 Vision:0];
    byte[2] = (Byte)(contentData.length >> 8);//取高八位
    byte[3] = (Byte)(contentData.length & 0xff);//取低八位
    byte[4] = (Byte)(crc >> 8);//CRC 高八位
    byte[5] = (Byte)(crc & 0xff);//CRC 低八位
    byte[6] = (Byte)(sid >> 8);//序列号 高八位
    byte[7] = (Byte)(sid & 0xff);//序列号 低八位
    // 3.L2数据(5bytes+buffer) + L1
    NSMutableData *data = [[NSMutableData alloc] initWithBytes:byte length:8];
    [data appendData:contentData];
    // 4.构建请求体指令发送
    ECellBLEItem *bleItem = [[ECellBLEItem alloc] initWithbyteBuffer:data cmd:cmd cmdKey:cmdKey sid:sid resultBlock:result];
    SEL sel = NSSelectorFromString(@"writeWithBleItem:");
    PerformSelectorLeakWarning([[ECellBluetoothManager shareInstance] performSelector:sel withObject:bleItem]);
}

/// 序号
+ (NSInteger)getSendCmdID {
    static NSInteger nSendCmdID = 0;
    nSendCmdID ++;
    return nSendCmdID;
}

/**
 构造L1层第二个字节
 
 @param err    1:对的 0:错的
 @param ack    1:回复 0:命令
 @param vision 版本
 */
+ (Byte)stitchWithErr:(int)err Ack:(int)ack Vision:(int)vision {
    int a = 0 * pow(2, 7) + 0 * pow(2, 6) + err * pow(2, 5) + ack * pow(2, 4) + vision;
    return a;
}

#pragma mark ======= Set Command =======

/// 同步基础配置(时间/语言)跟随系统同步(默认连接上自动同步) | Synchronization basic configuration (platform/device information/time/language) follows system synchronization(Automatic synchronization on the default connection)
+ (void)syncConfig {
    // 设置时间/时制
    NSDate *nowDate = [NSDate date];
    NSCalendar *calendar = [NSCalendar currentCalendar];
    NSUInteger unitFlags = NSCalendarUnitYear | NSCalendarUnitMonth | NSCalendarUnitDay | NSCalendarUnitHour | NSCalendarUnitMinute | NSCalendarUnitSecond | NSCalendarUnitWeekday;
    NSDateComponents *dateComponent = [calendar components:unitFlags fromDate:nowDate];
    NSInteger year = [dateComponent year];
    NSInteger month = [dateComponent month];
    NSInteger day = [dateComponent day];
    NSInteger hour = [dateComponent hour];
    NSInteger minute = [dateComponent minute];
    NSInteger second = [dateComponent second];
    // 手环设置时间最后拼接一个12/24小时制，hasAMPM NO：24小时 YES：12小时
    NSString *formatStringForHours = [NSDateFormatter dateFormatFromTemplate:@"j" options:0 locale:[NSLocale currentLocale]];
    NSRange containsA = [formatStringForHours rangeOfString:@"a"];
    BOOL hasAMPM = containsA.location != NSNotFound;
    [self setDateCommandYear:year month:month day:day hour:hour min:minute second:second is12Hour:hasAMPM callback:nil];
//    // 设置语言
//    [self setSystemUserCommandLanguageCode:[self getCurrentLanguageCode] is12Hour:hasAMPM callback:nil];
}

/// 设置当前日期 | Set current date
/// @param year 年 | year
/// @param month 月 | month
/// @param day 日 | day
/// @param hour 时 | hour
/// @param min 分 | min
/// @param second 秒 | second
/// @param is12Hour 时间制 | time system
+ (void)setDateCommandYear:(NSInteger)year
                     month:(NSInteger)month
                       day:(NSInteger)day
                      hour:(NSInteger)hour
                       min:(NSInteger)min
                    second:(NSInteger)second
                  is12Hour:(BOOL)is12Hour
                  callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[7];
    buffer[0] = year % 2000;
    buffer[1] = month;
    buffer[2] = day;
    buffer[3] = hour;
    buffer[4] = min;
    buffer[5] = second;
    buffer[6] = is12Hour;
    NSData *param = [NSData dataWithBytes:buffer length:7];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_setIngTime ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 系统用户设置 | Set System user Settings
/// @param languageCode 0:中文 1:英文 2.俄罗斯语 3.西班牙语 4.德国 5.意大利 6.法国 7.葡萄牙 8.波兰语 9.荷兰 10.希腊 11.土耳其 12.罗马尼亚 13.日语 14.繁体（目前只支持M6） 15.希伯来语 16.丹麦语 17.塞尔维亚语 18:其他 | 0: Chinese 1: English 2. Russian 3. Spanish 4. Germany 5. Italy 6. France 7. Portugal 8. Polish 9. Netherlands 10. Greece 11. Turkey 12. Romania 13. Japanese 14. Traditional (currently only supported M6) 15. Hebrew 16. Danish 17. Serbian 18: others
/// @param is12Hour 时间制 | time system
+ (void)setSystemUserCommandLanguageCode:(NSInteger)languageCode
                                is12Hour:(BOOL)is12Hour
                                callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[4];
    buffer[0] = languageCode;
    buffer[1] = is12Hour;
    buffer[2] = 0;
    buffer[3] = 1;
    NSData *param = [NSData dataWithBytes:buffer length:4];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_setSystemUser ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 用户信息设置 | Set User information Settings
/// @param gender 0:女 1:男 | 0: female 1: male
/// @param age 年龄 | age
/// @param height 身高（cm） | height(cm)
/// @param weight 体重（kg） | weight(kg)
/// @param goal 运动目标（步数） | Moving target (steps)
/// @param BMI BMI
+ (void)setUserInfoCommandGender:(NSInteger)gender
                             age:(NSInteger)age
                          height:(NSInteger)height
                          weight:(NSInteger)weight
                            goal:(NSInteger)goal
                             BMI:(NSInteger)BMI
                        callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[9];
    buffer[0] = gender;
    buffer[1] = age;
    buffer[2] = height;
    buffer[3] = weight;
    buffer[4] = goal;
    buffer[5] = goal >> 8;
    buffer[6] = goal >> 16;
    buffer[7] = 0;
    buffer[8] = BMI;
    NSData *param = [NSData dataWithBytes:buffer length:9];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_setUserInfo ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 设置推送 | Set the push
/// @param pushValue 推送值 0:推送所有 1:关闭推送（bit0：来电 bit1：短信 bit2：QQ bit3: 微信 bit4：whatsapp bit5：Messenger bit6：Twitter bit7：linkedin bit8：Instagram bit9：facebook bit10：others bit11:line bit12:viber bit13:skype bit14:Outlook）|
/// push value 0:push all 1:close push(Bit0: call bit1: SMS bit2: QQ bit3: WeChat bit4: whatsapp bit5: Messenger bit6: Twitter bit7: linkedin Bit8: Instagram bit9: facebook bit10: others bit11:line bit12:viber bit13:skype bit14:Outlook)
+ (void)setPushCommandPushValue:(NSInteger)pushValue
                       callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    NSData *param = [NSData dataWithBytes:&pushValue length:2];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_setPush ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 设置心率检测 | Set heart rate detection
/// @param isOpen 开关 | switch
/// @param startHour 开始时 | start hour
/// @param startMin 开始分 | start min
/// @param endHour 结束时 | end hour
/// @param endMin 结束分 | end min
/// @param interval 间隔（分钟）10的倍数 | interval（min）
+ (void)setHeartRateCheckCommandIsOpen:(BOOL)isOpen
                             startHour:(NSInteger)startHour
                              startMin:(NSInteger)startMin
                               endHour:(NSInteger)endHour
                                endMin:(NSInteger)endMin
                              interval:(NSInteger)interval
                              callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[6];
    buffer[0] = isOpen;
    buffer[1] = startHour;
    buffer[2] = startMin;
    buffer[3] = endHour;
    buffer[4] = endMin;
    buffer[5] = interval;
    NSData *param = [NSData dataWithBytes:buffer length:6];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_heartRateCheck ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 设置勿扰模式 | Set do not disturb mode
/// @param isOpen 开关 | switch
/// @param startHour 开始时 | start hour
/// @param startMin 开始分 | start min
/// @param endHour 结束时 | end hour
/// @param endMin 结束分 | end min
+ (void)setNotDisturbCommandIsOpen:(BOOL)isOpen
                             startHour:(NSInteger)startHour
                              startMin:(NSInteger)startMin
                               endHour:(NSInteger)endHour
                                endMin:(NSInteger)endMin
                          callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[5];
    buffer[0] = isOpen;
    buffer[1] = startHour;
    buffer[2] = startMin;
    buffer[3] = endHour;
    buffer[4] = endMin;
    NSData *param = [NSData dataWithBytes:buffer length:5];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_notDisturb ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 设置喝水提醒 | Set a water reminder
/// @param isOpen 开关 | switch
/// @param startHour 开始时 | start hour
/// @param startMin 开始分 | start min
/// @param endHour 结束时 | end hour
/// @param endMin 结束分 | end min
/// @param cycle 重复 0:表示仅一次 1:表示每天重复（bit0 - bit6分别表示周一到周日的重复设置，所有位都为0时，表示只提醒一次） | Repeat 0: only once 1: every day (bit0-bit6 is the repeat setting from Monday to Sunday, and when all bits are 0, it is only reminded once)
/// @param interval 间隔（分钟）| interval（min）
+ (void)setDrinkRemindCommandIsOpen:(BOOL)isOpen
                          startHour:(NSInteger)startHour
                           startMin:(NSInteger)startMin
                            endHour:(NSInteger)endHour
                             endMin:(NSInteger)endMin
                              cycle:(NSInteger)cycle
                           interval:(NSInteger)interval
                           callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[8];
    buffer[0] = isOpen;
    buffer[1] = startHour;
    buffer[2] = startMin;
    buffer[3] = endHour;
    buffer[4] = endMin;
    buffer[5] = cycle;
    buffer[6] = interval & 0xff;
    buffer[7] = interval >> 8;
    NSData *param = [NSData dataWithBytes:buffer length:8];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_setDrinkRemind ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 设置久坐提醒
/// @param isOpen 开关 | switch
/// @param startHour 开始时 | start hour
/// @param endHour 结束时 | end hour
/// @param cycle 重复 0:表示仅一次 1:表示每天重复（bit0 - bit6分别表示周一到周日的重复设置，所有位都为0时，表示只提醒一次） | Repeat 0: only once 1: every day (bit0-bit6 is the repeat setting from Monday to Sunday, and when all bits are 0, it is only reminded once)
/// @param sittingTime 久坐时间(分钟)超过这个时间提醒 | Sitting time (min) Over this time reminder
/// @param sittingThreshold 久坐阈值（步数）以小时为单位久坐时间内，步数低于这个阈值才提醒 | Sedentary threshold（Step）
+ (void)setSedentaryCommandIsOpen:(BOOL)isOpen
                        startHour:(NSInteger)startHour
                          endHour:(NSInteger)endHour
                            cycle:(NSInteger)cycle
                      sittingTime:(NSInteger)sittingTime
                 sittingThreshold:(NSInteger)sittingThreshold
                         callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[8];
    buffer[0] = isOpen;
    buffer[1] = startHour;
    buffer[2] = endHour;
    buffer[3] = cycle;
    buffer[4] = sittingTime;
    buffer[5] = sittingTime >> 8;
    buffer[6] = sittingThreshold & 0xff;
    buffer[7] = sittingThreshold >> 8;
    NSData *param = [NSData dataWithBytes:buffer length:8];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_setSedentary ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 设置闹钟 | Set the alarm clock
/// @param alarms 闹钟模型数组（最多5个）| Alarm clock model array (up to 5)
/// alarms: @[@{@"hour": Int, @"min": Int, @"cycle": Int, @"label": Int(0：闹钟 1：会议), @"isOpen": BOOL}]
+ (void)setAlarmClockCommandAlarms:(NSArray *)alarms
                          callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[25];
    for (int i = 0; i < alarms.count; i ++) {
        NSDictionary *alarm = alarms[i];
        NSInteger hour = [alarm[@"hour"] integerValue];
        NSInteger min = [alarm[@"min"] integerValue];
        NSInteger cycle = [alarm[@"cycle"] integerValue];
        NSInteger label = [alarm[@"label"] integerValue];
        NSInteger isOpen = [alarm[@"isOpen"] integerValue];
        buffer[0 + i * 5] = hour;
        buffer[1 + i * 5] = min;
        buffer[2 + i * 5] = cycle;
        buffer[3 + i * 5] = label;
        buffer[4 + i * 5] = isOpen;
    }
    NSData *param = [NSData dataWithBytes:buffer length:25];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_setAlarmClock ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 设置单位
/// @param measure 度量单位（0:公制 1:英制） | Unit of measure (0: metric 1: imperial)
/// @param temperature 温度（0: 摄氏度 1:华氏摄氏度）| Temperature (0: Celsius 1: Fahrenheit)
+ (void)setUnitCommandMeasure:(NSInteger)measure
                  temperature:(NSInteger)temperature
                     callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[2];
    buffer[0] = measure;
    buffer[1] = temperature;
    NSData *param = [NSData dataWithBytes:buffer length:2];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_unitSetting ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 设置运动目标 | Set a moving target
/// @param step 步数 | step
/// @param sportMode 0: 日常计步 1:走路 2: 跑步 3: 骑行 4: 爬山 | 0: daily step counting 1: walking 2: running 3: cycling 4: climbing
+ (void)setSportTargetCommandStep:(NSInteger)step
                        sportMode:(NSInteger)sportMode
                         callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[2];
    buffer[0] = step;
    buffer[1] = sportMode;
    NSData *param = [NSData dataWithBytes:buffer length:2];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_setSportTarget ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 天气推送 | Set weather push
/// @param lowTemperature 低温 | low temperature
/// @param highTemperature 高温 | hight temperature
/// @param weatherType 天气类型（0:晴 1:阴 2:雨 3:雪） | weather type(0: fine 1: Yin 2: rain 3: snow)
/// @param currenTemperature 当前温度 | curren temperature
+ (void)setWeatherPushCommandLowTemperature:(NSInteger)lowTemperature
                            highTemperature:(NSInteger)highTemperature
                                weatherType:(NSInteger)weatherType
                          currenTemperature:(NSInteger)currenTemperature
                                   callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[4];
    buffer[0] = lowTemperature;
    buffer[1] = highTemperature;
    buffer[2] = weatherType;
    buffer[3] = currenTemperature;
    NSData *param = [NSData dataWithBytes:buffer length:4];
    [self packetData:ECell_BLECMD_weather cmdKey:ECell_BLECMD_KEY_weatherPush ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}


/// 发送AQI数据
/// @param aqi aqi
/// @param humidity 湿度 （%）
/// @param windDirection 风向 0：无 1:北 2：东北 3：东 4：东南 5：南 6：西南 7：西 8：西北
/// @param windSpeed 风速 （公里/小时）
+ (void)setAQICommandWithAQI:(NSInteger)aqi
                    humidity:(NSInteger)humidity
               windDirection:(NSInteger)windDirection
                   windSpeed:(NSInteger)windSpeed
                    callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback
{
    Byte buffer[6];
    buffer[0] = aqi & 0xff;
    buffer[1] = aqi >> 8;
    buffer[2] = humidity;
    buffer[3] = windDirection;
    buffer[4] = windSpeed & 0xff;
    buffer[5] = windSpeed >> 8;
    NSData *param = [NSData dataWithBytes:buffer length:6];
    [self packetData:ECell_BLECMD_weather cmdKey:ECell_BLECMD_KEY_AQIPush ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
    
    
}


/// 设置提醒模式
/// @param remindType 提醒方式 1:亮屏 2:震动 3:亮屏+震动 | Reminder 1: bright screen 2: vibration 3: bright screen + vibration
+ (void)setRemindModelCommandRemindType:(NSInteger)remindType
                               callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[1];
    buffer[0] = remindType;
    NSData *param = [NSData dataWithBytes:buffer length:1];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_remindModel ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

#pragma mark ======= Get Command =======

/// 获取固件版本信息 | Get firmware information
+ (void)getFirmwareInfoCommand:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary *_Nullable json))callback {
    [self packetData:ECell_BLECMD_firmware cmdKey:ECell_BLECMD_KEY_firmwareInfo ParamData:nil ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType, json);
        }
    }];
}

/// 获取电量 | Get attery
+ (void)getBatteryCommand:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary *_Nullable json))callback {
    [self packetData:ECell_BLECMD_device cmdKey:ECell_BLECMD_KEY_battery ParamData:nil ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType, json);
        }
    }];
}

/// 获取配置信息 | Get configuration information
+ (void)getProfileCommand:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary *_Nullable json))callback {
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_readProfile ParamData:nil ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType, json);
        }
    }];
}

/// 获取当前实时数据 | Get the current real-time data
/// @param type 1：睡眠 2：心率 3：计步 4：运动模式 | 1: sleep 2: heart rate 3: count steps 4: exercise mode
+ (void)getRealTimeDataCommandType:(NSInteger)type
                          callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary *_Nullable json))callback {
    NSData *param = [NSData dataWithBytes:&type length:1];
    [self packetData:ECell_BLECMD_sync cmdKey:ECell_BLECMD_KEY_realTimesync ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType, json);
        }
    }];
}

/// 获取指定日期健康数据 | Gets health data for the specified date
/// @param type 1：睡眠，2：心率，3：计步，4：运动 5:血压 6: 心电 7: 血氧 8: 体温 | 1: sleep, 2: heart rate, 3: step counting, 4: exercise 5: blood pressure 6:electrocardio 7: blood oxygen 8: body temperature
/// @param year 年
/// @param month 月
/// @param day 日
/// 设备最多保存七天数据 | The device holds data for up to seven days
+ (void)getHealthDataCommandType:(NSInteger)type
                            year:(NSInteger)year
                           month:(NSInteger)month
                             day:(NSInteger)day
                        callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary *_Nullable json))callback {
    Byte buffer[7];
    buffer[0] = type;
    buffer[1] = year > 2000 ? year % 100 : year;
    buffer[2] = month;
    buffer[3] = day;
    NSData *param = [NSData dataWithBytes:buffer length:7];
    [self packetData:ECell_BLECMD_sync cmdKey:ECell_BLECMD_KEY_syncHealth ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType, json);
        }
    }];
}

#pragma mark ======= Control Command =======

/// 查找设备 | Find device
+ (void)findDeviceCommand:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    [self packetData:ECell_BLECMD_find cmdKey:ECell_BLECMD_KEY_findDevice ParamData:nil ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 抬手亮屏 | Raised my hand against the bright screen
/// @param hand 0:左手 1:右手 | 0: left hand 1: right hand
/// @param raise 0:抬手亮屏关闭  1:抬手亮屏开启 | 0: turn off the bright screen 1: turn on the bright screen
/// @param flip 0:翻腕亮屏关闭  1:翻腕亮屏开启 | 0: turn wrist bright screen off 1: turn wrist bright screen on
+ (void)gesturesControlCommandHand:(NSInteger)hand
                             raise:(NSInteger)raise
                              flip:(NSInteger)flip
                          callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[3];
    buffer[0] = hand;
    buffer[1] = raise;
    buffer[2] = flip;
    NSData *param = [NSData dataWithBytes:buffer length:3];
    [self packetData:ECell_BLECMD_device cmdKey:ECell_BLECMD_KEY_handUp ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 远程拍照控制 | Remote camera control
/// @param isOpen 0:相机已关闭 1:相机已打开 | 0: camera closed 1: camera on
+ (void)cameraControlCommandIsOpen:(NSInteger)isOpen
                          callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    NSData *param = [NSData dataWithBytes:&isOpen length:1];
    [self packetData:ECell_BLECMD_device cmdKey:ECell_BLECMD_KEY_camera ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

#pragma mark ======= Listen Command =======

/// 设备主动上报蓝牙数据 | The device actively reports bluetooth data
+ (void)listenDevceReportCommand:(void(^_Nullable)(ECell_BLE_REPORT_COMMAND reportCmd, NSDictionary *_Nullable json))callback {
    // 设置监听
    ResultReportBlock result = callback;
    SEL sel = NSSelectorFromString(@"listenResult:");
    PerformSelectorLeakWarning(
                               [[ECellBluetoothManager shareInstance] performSelector:sel withObject:result]);
}

#pragma mark ======= Send Flash Command =======

/// FLASH数据发送命令 | FLASH data send command
/// @param total 总包数
/// @param curren 当前包序号
/// @param lenght 数据长度
/// @param data 数据
+ (void)sendFlashDataCommandTotal:(NSInteger)total
                           curren:(NSInteger)curren
                           lenght:(NSInteger)lenght
                             data:(NSData *)data
                         callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    NSInteger crc = 0xff;
    for (int i = 0; i < data.length; i++) {
        crc^= ((Byte*)([data bytes]))[i] & 0xFF;
        for (int j = 0; j < 8; j++) {
            if((crc & 1) !=0){
                crc >>=1;
                crc^=0xB8;
            }else{
                crc>>=1;
            }
        }
    }
    NSMutableData *param = [NSMutableData dataWithBytes:&total length:2];
    [param appendData:[NSData dataWithBytes:&curren length:2]];
    [param appendData:[NSData dataWithBytes:&lenght length:2]];
    [param appendData:[NSData dataWithBytes:&crc length:2]];
    [param appendData:data];
    [self packetData:ECell_BLECMD_sendFlash cmdKey:ECell_BLECMD_KEY_falshSend ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

/// 表盘推送启动 | Dial push start
/// @param number 第几个表盘 | number dial
/// @param type 0:删除表盘 1:增加表盘 2:更新表盘 | 0: delete dial 1: add dial 2: update dial
/// @param fileLenght 文件大小 | The file lenght
+ (void)sendDialPushCommandNumber:(NSInteger)number
                            type:(NSInteger)type
                      fileLenght:(NSInteger)fileLenght
                         callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback {
    Byte buffer[2];
    buffer[0] = number;
    buffer[1] = type;
    NSMutableData *param = [NSMutableData dataWithBytes:buffer length:2];
    [param appendData:[NSData dataWithBytes:&fileLenght length:4]];
    [self packetData:ECell_BLECMD_dialPush cmdKey:ECell_BLECMD_KEY_dialPush ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}

#pragma mark ======= Tool =======

+ (NSInteger)pushTypeSwitchWithIsOpen:(BOOL)isOpen BitIndex:(NSInteger)bitIndex State:(NSInteger)state {
    state = isOpen ? (1<<bitIndex) | state : ~(1<<bitIndex) & state;
    return state;
}

#pragma mark ======= 透传数据 =======


/// 透传数据
/// @param paramData 参数数据
/// @param callback 回调结果
+ (void)customParamData:(nullable NSData *)paramData
               callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary *_Nullable json))callback
{
    [self packetData:0x14 cmdKey:0x20 ParamData:paramData ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if(json.count > 0)
        if (callback && json) {
            callback(errorType, json);
        }
    }];
}


/// 恢复出厂设置
/// @param callback 回调结果
+ (void)restoreFactoryCommand:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback
{
    [self packetData:ECell_BLECMD_factory cmdKey:ECell_BLECMD_KEY_restoreFactory ParamData:nil ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}


/// 睡眠设置
/// @param ONOFF 0:关闭 1:开启
/// @param startHour 开始时间(时分秒,格式如：221000：晚上22点10分00秒)
/// @param endHour 结束时间(时分秒,格式如：061000：早上6点10分00秒)
/// @param callback 回调结果
+ (void)setSleepCommandWithONOFF:(BOOL)ONOFF
                       startHour:(NSInteger)startHour
                         endHour:(NSInteger)endHour
                        callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback;
{
    Byte buffer[7];
    buffer[0] = ONOFF;
    buffer[1] = startHour;
    buffer[2] = endHour;
    NSData *param = [NSData dataWithBytes:buffer length:7];
    [self packetData:ECell_BLECMD_setUp cmdKey:ECell_BLECMD_KEY_setSleep ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}


/// 解除绑定
/// @param callback 回调结果
+ (void)rescindDindingCommand:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback
{
    [self packetData:ECell_BLECMD_device cmdKey:ECell_BLECMD_KEY_unbinding ParamData:nil ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}


///  气象指数推送
/// @param UV 紫外线
/// @param pressure 气压
/// @param altitude 海拔
/// @param callback 回调结果
+ (void)weatherIndexWithCommandWithUV:(NSString *)UV
                          Pressure:(NSString *)pressure
                          Altitude:(NSString *)altitude
                             callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback
{
    Byte buffer[5];
    buffer[0] = [UV intValue];
    buffer[1] = [pressure intValue] & 0xff;
    buffer[2] = [pressure intValue] >> 8;
    buffer[3] = [altitude intValue] & 0xff;
    buffer[4] = [altitude intValue] >> 8;
    NSData *param = [NSData dataWithBytes:buffer length:5];
    [self packetData:ECell_BLECMD_pushDevice cmdKey:ECell_BLECMD_KEY_weatherIndex ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}


/// 固件升级启动
/// @param updateData 固件升级包数据
/// @param callback 回调结果
+ (void)firmwareUpdateWithUpdateData:(NSData *)updateData
                            callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback
{
    [self packetData:ECell_BLECMD_firmware cmdKey:ECell_BLECMD_KEY_firmwareUpdate ParamData:updateData ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}


/// 设备绑定
/// @param uid 用户ID
/// @param mscMsg MAC信息
/// @param callback 回调结果
+ (void)setDeviceBindingWithUid:(NSString *)uid
                         MacMsg:(NSString *)mscMsg
                       callback:(void(^_Nullable)(ECell_BLE_COMMAND_ERROR_TYPE errorType))callback
{
    NSMutableData *param = [NSMutableData new];
    NSData *uiddata = [uid dataUsingEncoding:NSUTF16LittleEndianStringEncoding];
    NSData *mscMsgdata = [mscMsg dataUsingEncoding:NSUTF8StringEncoding];
    
    unsigned long len = [uiddata length];
    [param appendBytes:&len length:6];
    [param appendData:uiddata];
    
    unsigned long len1 = [mscMsgdata length];
    [param appendBytes:&len1 length:6];
    [param appendData:mscMsgdata];
    
    [self packetData:ECell_BLECMD_device cmdKey:ECell_BLECMD_KEY_binding ParamData:param ResultBlock:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
        if (callback) {
            callback(errorType);
        }
    }];
}




/**
 将周期数组转成INT

 @param cycle 周期数组 @[@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0"]
 @return 转换后的INT
 */
+ (NSInteger)conversionCycleArray:(NSArray *)cycle {
    if (!cycle || cycle.count != 8) {
        return 0;
    }
    NSArray* reversedArray = [[cycle reverseObjectEnumerator] allObjects];
    NSInteger cycleInt = 0;
    for (NSInteger i = 0; i < reversedArray.count; i ++) {
        NSInteger indexNumber = [reversedArray[i] integerValue];
        if (indexNumber) {
            cycleInt += 1 * pow(2, i);
        }
    }
    return cycleInt;
}

/**
 返回当前语言代码
 */
+ (NSInteger)getCurrentLanguageCode {
    NSString *language = [[NSLocale preferredLanguages] objectAtIndex:0];
    if ([language hasPrefix:@"zh"]) {
        //  单独处理有繁体的手环设备
        if ([language hasPrefix:@"zh-Hant"]) {
            return 14;
        }
        return 0;
    }else if ([language hasPrefix:@"en"]) {
        return 1;
    }else if ([language hasPrefix:@"ru"]) {
        return 2;
    }else if ([language hasPrefix:@"es"]) {
        return 3;
    }else if ([language hasPrefix:@"de"]) {
        return 4;
    }else if ([language hasPrefix:@"it"]) {
        return 5;
    }else if ([language hasPrefix:@"fr"]) {
        return 6;
    }else if ([language hasPrefix:@"pt"]) {
        return 7;
    }else if ([language hasPrefix:@"pl"]) {
        return 8;
    }else if ([language hasPrefix:@"nl"]) {
        return 9;
    }else if ([language hasPrefix:@"el"]) {
        return 10;
    }else if ([language hasPrefix:@"tr"]) {
        return 11;
    }else if ([language hasPrefix:@"ro"]) {
        return 12;
    }else if ([language hasPrefix:@"ja"]) {
        return 13;
    }else if ([language hasPrefix:@"he"]) {
        return 15;
    }else if ([language hasPrefix:@"da"]) {
        return 16;
    }else if ([language hasPrefix:@"sr"]) {
        return 17;
    }else {
        return 18;
    }
}

@end
